/* -------------------------------------------------------------------
  Titre: Serveur HTTP
  Date 19/05/2021
  Objectifs:
  Commander la led L et la sortie D0 d'une carte MKR Wifi 1010,
  et obtenir les valeurs présentes sur les entrées A0 et A1 à partir
  d'un navigateur (requêtes GET).
  URL reconnues:
  - http://@IP renvoie "La page d'accueil"
  - http://@IP/arduino/digital/led/0 active la LED et D0
  - http://@IP/arduino/digital/led/1 désactive la LED et D0
  - http://@IP/arduino/analog/val1 demande la valeur sur A0
  - http://@IP/arduino/analog/val2et1 demande les valeurs sur A0 et A1
  Carte arduino:
  - Module NINA (Arduino MKR WiFi 1010, MKR VIDOR 4000 et UNO WiFi Rev.2)
  -----------------------------------------------------------------------*/
// Définitions
#define LED LED_BUILTIN //  LED_BUILTIN (L sur la carte !)
#define D0 0

// Bibliothèques
#include <SPI.h>
#include <WiFiNINA.h>

// Paramètres réseau, cryptage WPA
const char ssid[] = "à compléter";   // nom du SSID
const char pass[] = "à compléter"; // mot de passe du réseau

// Page d'accueil
const char paccueil[] = "<html><head><meta charset=\"utf-8\">"
                        "<title>Serveur MKR Wifi 1010</title>"
                        "<style>"
                        ".ip{color:blue;}</style>"
                        "</head><body>"
                        "<h3>Page d\'accueil du serveur sur MKR Wifi 1010</h3>"
                        "<strong>Commandes reconnues:</strong><br>"
                        "- http://@IP<span class=\"ip\"><a href=\"/arduino/digital/led/1\">/arduino/digital/led/1</a></span> active la LED L et la sortie D0<br>"
                        "- http://@IP<span class=\"ip\"><a href=\"/arduino/digital/led/0\">/arduino/digital/led/0</a></span> désactive la LED L et la sortie D0<br>"
                        "- http://@IP<span class=\"ip\"><a href=\"/arduino/analog/val1\">/arduino/analog/val1</a></span> renvoie la valeur sur l'entrée A0<br>"
                        "- http://@IP<span class=\"ip\"><a href=\"/arduino/analog/val2et1\">/arduino/analog/val2et1</a></span> renvoie les valeurs sur les entrées A0 et A1 -> val2 au format JSON<br>"
                        "</body></html>";
// Client
WiFiClient client;

// Fonctions
// Formatage de la réponse HTTP
void reponse(String message)
{
  client.println("HTTP/1.1 200 OK");
  client.println("Content-type:text/html");
  client.println();
  client.println(message);
  client.println();
}

// Mesure et transmission des valeurs sur A0 et A1
void handle2Measures()
{
  unsigned int val1 = analogRead(0);
  unsigned int val2 = analogRead(1);
  String msgjson = "[{\"val1\":";
  msgjson += val1;
  msgjson += ",";
  msgjson += "\"val2\":";
  msgjson += val2;
  msgjson += "}]";
  reponse(msgjson);
}

// Initialisations
// -------------------------------------------------------------------
int status = WL_IDLE_STATUS;
// Création du serveur, écoute sur le port 80
WiFiServer server(80);

void setup()
{
  // Configuration des E/S
  pinMode(LED, OUTPUT); // Led L de la carte
  pinMode(D0, OUTPUT);
  Serial.begin(115200); // Moniteur pour la mise au point

  // 1. Vérification de la connexion au module Wifi
  if (WiFi.status() == WL_NO_MODULE)
  {
    Serial.println(" La communication avec le module WiFi a échoué!");
    // Blocage du programme !!!!
    while (true)
      ;
  }

  // 2. Connexion au réseau Wifi
  while (status != WL_CONNECTED)
  {
    Serial.print("Tentative de connexion au réseau: ");
    Serial.println(ssid);
    status = WiFi.begin(ssid, pass); // si WPA, WPA2
    // Attente de 10s avant la prochaine tentative de connexion
    delay(10000);
  }

  // 3. Connecté, alors démarrage du serveur Web sur le port défini
  // lors de l'initialisation
  server.begin();
}
// -------------------------------------------------------------------
void loop()
{ // 4. Traiter les requêtes
  client = server.available(); // A l'écoute (blocante) d'un client

  if (client)
  { // Client connecté !
    Serial.println("Nouveau client !!!");
    String currentLine = ""; // Création d'une chaîne pour mémoriser
    // les données transmises par le client
    while (client.connected())
    {
      if (client.available())
      { // Si il y a des octets à lire
        char c = client.read(); // on les lit !!!!
        if (c == '\n') // si le dernier caractère lu est une fin de ligne
        {
          if (currentLine.length() == 0) // si la ligne est vide
          { // c'est la fin de la requête HTTP du client, alors on envoie la page d'accueil:
            // REMARQUE : les en-têtes HTTP commencent toujours par un code de réponse
            // (par exemple HTTP / 1.1 200 OK)
            // et un type de contenu pour que le client sache ce qui lui est envoyé, puis une ligne vide :
            // Page d'accueil
            reponse(paccueil);
            // On sort de la boucle while après avoir envoyé la réponse
            break;
          }
          else
          { // si nouvelle ligne, on vide currentLine:
            currentLine = "";
          }
        }
        else if (c != '\r') // si le dernier caractère lu n'est pas un retour chariot,
        {
          currentLine += c; // on l'ajoute à la fin de currentLine
        }
        // Traitement des requêtes (commence par GET)
        // ----------------------------------------------------
        // Requête 1
        if (currentLine.endsWith("GET /arduino/digital/led/1"))
        {
          digitalWrite(LED, HIGH); // Activation de la commande de la LED
          digitalWrite(D0, HIGH);  // et de la sortie D0
          reponse(String("1"));
          // On sort de la boucle while après avoir envoyé la réponse
          break;
        }
        // Requête 2
        if (currentLine.endsWith("GET /arduino/digital/led/0"))
        {
          digitalWrite(LED, LOW); // Désactivation de la commande de la LED
          digitalWrite(D0, LOW);  // et de la sortie D0
          reponse(String("0"));
          // On sort de la boucle while après avoir envoyé la réponse
          break;
        }
        // Requête 3
        if (currentLine.endsWith("GET /arduino/analog/val1"))
        {
          unsigned int val1 = analogRead(0);
          reponse(String(val1));
          // On sort de la boucle while après avoir envoyé la réponse
          break;
        }
        // Requête 4
        if (currentLine.endsWith("GET /arduino/analog/val2et1"))
        {
          handle2Measures(); // Lit les entrées A0,A1 et
          // les transmet au format JSON
          // On sort de la boucle while après avoir envoyé la réponse
          break;
        }
        // Fin traitement des requêtes
        // ----------------------------------------------------
      } // fin si tous les octet sont lus
    } // fin tant que
    // Fermeture de la connexion
    client.stop();
    Serial.println("Client déconnecté");
  }
}
